#!/bin/bash

#	Script name : RestTime
#	Programmer : mazKnez 
#	Website : mazKnez.com
#	2019-08-17


########################### [-- Readme ################################

# This script has been tested on Debian-based Linux distributions.
# NOTICE : Check that these packages are installed on your system : " beep, pidof, speaker-test, pkill " 
# NOTICE : config.cfg should be next to the script file. 
# NOTICE : This script must be run with root access.
# NOTICE : If you want to run this script automatically (via service or in rc.local), 
# 	       you need to include in the file ".profile" the following commands to access the monitor:	
#	       xhost +SI:localuser:root > /dev/null 
#  		   OR
#		   xhost + > /dev/null 		
# 
#
# This script calculates how long you will work with PC and reminds you after a certain amount of time
# 	that you need to take some rest(short break). 
# This script automatically calculates the working time of the PC and the script does not take into
# 	account the times when you does not work with the PC(idle time). 
# When you work with computers for a long time, this script is essential for your health.	
# 
#
# Please read "README" file or visit www.mazKnez.com website for more information.
#
########################### Readme --] ################################



## [-- define consts   

readonly script_path=$(dirname "${0}")"/"
readonly keyboards_file="${script_path}keyboards_dump"		# The keys pressed from the keyboard
															# are stored in this file.

readonly mice_file="${script_path}mice_dump"		# Mouse movements and clicks 
													# are stored in this file.

readonly status_file="${script_path}status"			# The script status is stored in this file, 
													# mostly used to verify the script. 

readonly config_file="${script_path}config.cfg"		# The script configuration is read from this file
export DISPLAY=:0 

## define consts --]  


main() {
   
	check_run_as_root		# Check script execution with super user
	if detect_script_running; then		# This script should not run multiple times at same time
	   echo "Script is runnig ..." 
	   exit 1 
	fi
	load_variables		# Load variables from configuration file
	dump_keyboard_and_mouse		# Store keyboards and mice activity in the files

	while true 		# Run the script forever
	do

		idleCounter=0‍‍	
		whileCounter=0

		while ((whileCounter < workingTime));		# This loop runs every minute until the end of workingTime.
		do
			load_variables		
			if "${active}"; then		# This variable loads from configuration file
				echo "You have" $((workingTime-whileCounter)) "minutes to work ... " > "$status_file"
				if is_last_minute; then		# Detect last minute of working time
					echo > "${keyboards_file}"		# Clear keyboards_file 
					play_notice_sound 1		# Play notice sound, The user can request more time to work
											# after hearing this sound
					for forCounter in {1..7}		# Wait a short time for the user to request extra time
					do 
						sleep 2s
						if check_more_time_requested; then		# Did the user request more time?
							((whileCounter-=2))		# Add 2 minutes to user deadline
							play_notice_sound 2		# Play audio to accept user request
						   break			
						fi
				    done
				fi

				sleep 1m		# Wait for 1 minutes									

				if "${idleDetection}"; then		# If IdleDetection is enable in the configuration file
					if is_user_working; then
						((whileCounter++))		# If the user is working, reduce one minute of working time.
						idleCounter=0
					else
						((idleCounter++))
					fi
					if ((idleCounter >= restingTime)); then
						# If the user does not work for 5 minutes then reset all the counters.
						# This means that the user has not been with the system for a while,
						# so, the user has enough rest and can work again.
						whileCounter=0
						idleCounter=0
					fi
				else		# If IdleDetection is disable. static mode
					idleCounter=0
				   ((whileCounter++))
				fi					
			else
			   idleCounter=0
			   whileCounter=0
			   sleep 1m  
			fi	

		done

		echo "It's time to rest !" > "${status_file}"
		end_work		# Executing commands related to the end of working time
		sleep "${restingTime}m"		 
		start_work		# Executing commands related to the start working time

	done

}


########################### [--- Functions ################################

check_run_as_root () {

  ### This function checks script execution with super user

  if ((UID != 0)); then  # root UID == 0
    echo "This script must run as root"
    exit 1
  fi	
}

detect_script_running () {

	### This function checks whether another version of this script is running
	### The best way to determine if software is running is to use a pidfile... 
	### But it is also applicable to this script.

	script_name=$(basename "${0}")
	is_another_script_running=$(pidof -x "${script_name}" -o $$)
	# pidof checks if the program is running under this name
	# Parameter -x was used to check the scripts.
	# Parameter -o is used to delete the current execution of the script from the list.
	if [ "${is_another_script_running}" ]; then
	    true
	else
		false
	fi
}

dump_keyboard_and_mouse () {

	### This function stores keyboards and mice activity in the files.
	### To detect user idle, we need to read the mouse and keyboard inputs. 
	###		We use the method of directly reading files in "/dev/input". 
	###		Along the way you can find the mice file and the keyboards file that starts with "event".
	### 	"mice" contains mouse input data, but to find the file related to the keyboards 
	###		we need to check the files in folder "by-path", keyboards file names end with "event-kbd". 
	###		We need to find the links to the keyboards, and then we can find the keyboards event file

	pkill --full "cat /dev/input/"		# Kill previous execution of this function

	kbdEvents=($(ls /dev/input/by-path | grep "event-kbd"))		# Find files related to keyboards
	for forCounter in "${kbdEvents[@]}"
	do
		eventFile=$(readlink --canonicalize "/dev/input/by-path/${forCounter}")		# Get keyboards file location
		cat "${eventFile}" > "${keyboards_file}" &		# Store keyboards activity in the keyboards_file
	done
	cat /dev/input/mice > "${mice_file}" &		# Store mice activity in the mice_file
}

load_variables () {	
	
	### This function loads variables from configuration file	
	### User can change the script behavior by these variables. 
	### NOTICE : This file(config.cfg) should be next to the script file.

	if [ -f "${config_file}" ]; then		# If config.cfg is extist
		source "${config_file}"
	else
		workingTime=30      # In minutes
		restingTime=5       # In minutes
		idleDetection=true      # If "true", the script also check idle time,
								# for more information about it, 
								# read "is_user_working" function's comments
		active=true     # If "false", The script is temporarily disabled.
		playSound=true      # If "false", the script do tasks silently.
	fi	
}


is_last_minute () {

  ### This function checks whether this minute is the last minute of working time.

  if ((whileCounter == workingTime-1)) && ((idleCounter == 0 )); then 
  		true
  else 
  		false
  fi
}

check_more_time_requested (){

	### This function checks whether the user has pressed the ScrollLock key 3 or more times. 
	### The user can press more than 2 times the ScrollLock key, demanding more time to work.

    scroll_lock_pressed=$(xxd "${keyboards_file}" | grep --count "0100 4600 0100 0000") 
    # xxd displays the input as hex.
    # Hex code of ScrollLock in "keyboards_file" is "0100 4600 0100 0000"
	if ((scroll_lock_pressed >= 3)); then 
		true # return true
	else
		false
	fi	
}

play_sound () { 
   
   ### This function plays sound through the speakers and the built-in speaker system.
   ### $1 == Frequency , $2 == Time in ms

   if "${playSound}"; then		# User can disable sound by changing the configuration file
		second=$(bc --mathlib <<< "$2/1000")		
		# Decimal numbers cannot be calculated by bash. 
		# We have to use another programs like bc to divide a decimal.
		beep -f "${1}" -l "${2}" < /dev/null&		# Play sound by built-in speaker system
		( speaker-test --test sine --frequency "${1}" ) > /dev/null& pid=$! ; sleep "${second}s" ; kill -9 "${pid}" 
	fi
}

play_notice_sound () {	
	
	### This function plays notice sounds	 

    if (("${1}"==1)); then		# $1 is for last minute notice
		for i in 1 2
		do 
			sleep 0.5s
			play_sound 800 80
			sleep 0.2s
			play_sound 800 80
		done
	fi
    if (("${1}"==2)); then		# $2 is for accept user request more time 	
		play_sound 1000 80
		sleep 0.2s
		play_sound 1200 100
	fi
}

is_user_working () {
	
	### This function checks whether the user has been working with the system or not.
	### This function checks for both keyboards and mice files, 
	### if their size is zero, it means the user is not working with the system.

	miceFileSize=$(stat --format %s "${mice_file}")		# Get mice file size
	kbdFileSize=$(stat --format %s "${keyboards_file}")		# Get keyboards file size

	echo > "${mice_file}" > "${keyboards_file}"		# Clear mice and keyboards files

	if ((miceFileSize > 1 )) || ((kbdFileSize > 1 )); then
		true # return true
	else
		false
	fi
}

end_work () {
	
	### This function Execute commands related to the end of working time 

	play_sound 800 500
	play_sound 700 500
	sleep 5s
	xset -display "${DISPLAY}" dpms force off		# Turn off the monitor
}

start_work () {
	
	### This function Execute commands related to the start working time 	

	play_sound 1000 400
	play_sound 1200 500
	play_sound 1200 700
	xset -display "${DISPLAY}" dpms force on 		# Turn on the monitor
}


########################### Functions --] ################################

main "${@}"  # Start main function
